from django.db import models
from reqman.apps.account.models import User

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

MAIN_CHOICES = ( ("",""),   ("M1", "M1 : system/function shall [not] set [<quantifier>] item [to stateValue]"), 
                            ("M2", "M2 : system/function shall [not] set state to stateValue"), 
                            ("M3", "M3 : system/function shall [not] send [<quantifier>] item"),
                            ("M4", "M4 : system/function shall [not] receive [<quantifier>] item"),
                            ("M5", "M5 : system/function shall [not] ingest flow"),
                            ("M6", "M6 : system/function shall [not] emit flow"),
                            ("M7", "M7 : system shall [not] perform function"),
                            ("M8", "M8 : function shall [not] invoke function"),
                            ("M9", "M9 : system shall [not] present interface [to system]"),
                            ("M10", "M10 : system/interface shall [not] transfer flow/item"),
                            ("M11", "M11 : system shall [not] interact with system [using <connection>] [as dened in <text>]"),
                            ("M12", "M12 : system shall [not] have state state [with values stateSet]"),
                            ("M13", "M13 : system state stateValue shall [not] have substate state [with values stateSet]"),
                            ("M14", "M14 : system item shall [not] take values from stateSet"),
                            ("M15", "M15 : system item shall [not] be composed from item"),
                            ("M16", "M16 : system shall [not] contain system")
                        )


def create_main(instance):
    '''Create the Main Model for the Boilerplate'''
    try :
        main = Main.objects.get(boilerplate_of_main = instance, prefix_owner_of_main = instance.owner)
        if instance.has_main:
            return
        main = Main.objects.get(boilerplate_of_main = instance, prefix_owner_of_main = instance.owner)
        main.delete()
    except Main.DoesNotExist:
        if instance.has_main:
            main = Main(boilerplate_of_main = instance, prefix_owner_of_main = instance.owner)
            main.save()




class Main(models.Model):
    boilerplate_of_main = models.ForeignKey('reqtool.Boilerplate', related_name='main', on_delete=models.CASCADE)
    prefix_owner_of_main = models.ForeignKey(User, related_name='main_owner', on_delete=models.CASCADE)
    main_choices = models.CharField(choices=MAIN_CHOICES, max_length=100)

    

    def __init__(self, *args, **kwargs):
        super(Main, self).__init__(*args, **kwargs)
        self.__important_fields = ['main_choices']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))
        

    def save(self, **kwargs):
        """Create P1, P2 or P3 """
        prev_main_list = []
        super(Main, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_main = getattr(self, orig)
            prev_main_list.append(previous_main)        
        '''Create Main '''
        #create_M1( self, self.simple_prefix_1, prev_prefix_list[0], '1')
           

    class Meta:
        ordering = ['prefix_owner_of_main']